//#include <ButtonDebounce.h>

#include <Bounce2.h>

#include <Adafruit_NeoPixel.h>

const int NUMPIXELS = 4;
const int BUTTONPIN = 4;
const int NEOPIXELPIN = 6;
const int VIBRATIONPIN = 5;

char val; // data received from serial port
int currTime = 0;
bool isVibrating = false;
bool isUnhappyLed = false;
bool isHappyLed = false;
int vibrationInterval = 200;
int vibrationRestInterval = 50;
int vibrationDuration = 2000;
int vibartionStartTime = 0;
int unhappyBlinkInterval = 200;
int ledStartTime = 0;

Bounce2::Button button = Bounce2::Button();
Adafruit_NeoPixel leds = Adafruit_NeoPixel(NUMPIXELS, NEOPIXELPIN, NEO_RGB + NEO_KHZ800); 

void setup() {
  pinMode(VIBRATIONPIN, OUTPUT);
  digitalWrite(VIBRATIONPIN, LOW);
  button.attach(BUTTONPIN, INPUT_PULLUP);
  button.interval(0);
  button.setPressedState(LOW); 
  leds.begin();
  leds.show();  // Initialize all pixels to 'off'
  Serial.begin(9600);
  randomSeed(analogRead(0));
}

void loop() {
  currTime = millis();
  checkVibration();
  checkHappyLED();
  checkUnHappyLed();
  val = getSerialData();
  if (val == 'v') {
    startVibration(); // vibration
  }
  if (val == 'h') {
    // happy LED
    startHappyLed();
  }
  if (val == 'i') {
    // unhappy LED
    startUnhappyLed();
  }
  if (isButtonPressed()) {
    // happy LED
    startHappyLed();
  }
}

void checkHappyLED() {
  if (!isHappyLed) return;
  int ledTime = currTime - ledStartTime;
  // blink twice first
  if (ledTime < (unhappyBlinkInterval*6)) {
    int c;
    if ((ledTime/unhappyBlinkInterval) % 2 == 0) {
      for(int i=0;i<NUMPIXELS;i++){
        c = random(3);
        if (c==0) leds.setPixelColor(i, leds.Color(255,0,0));
        if (c==1) leds.setPixelColor(i, leds.Color(0,255,0)); // colors
        if (c==2) leds.setPixelColor(i, leds.Color(0,0,255)); // colors
        leds.show(); // This sends the updated pixel color to the hardware.
      }
    }
    else {
      for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
      }
    }
  }
  else if (ledTime < (unhappyBlinkInterval*9)) {
    for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(255,255,255)); // white
        leds.show(); // This sends the updated pixel color to the hardware.
     }
  }
  else if (ledTime < (unhappyBlinkInterval*15)) {
     int x = (ledTime-unhappyBlinkInterval*9)/unhappyBlinkInterval%NUMPIXELS;
     for(int i=0;i<NUMPIXELS;i++){
        if (i == x) leds.setPixelColor(i, leds.Color(random(255),random(255),random(255))); // random
        else leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
     }
  }
  else {
    isUnhappyLed = false;
      ledStartTime = 0;
      for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
      }
  }
}

void checkUnHappyLed() {
  if (!isUnhappyLed) return;
  int ledTime = currTime - ledStartTime;
  // blink twice first
  if (ledTime < (unhappyBlinkInterval*4)) {
    if ((ledTime/unhappyBlinkInterval) % 2 == 0) {
      for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,255,0)); // red
        leds.show(); // This sends the updated pixel color to the hardware.
      }
    }
    else {
      for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
      }
    }
  }
  else if (ledTime < (unhappyBlinkInterval*6)) {
    for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,255,0)); // red
        leds.show(); // This sends the updated pixel color to the hardware.
     }
  }
  else if (ledTime < (unhappyBlinkInterval*9)) {
     int t = (ledTime-unhappyBlinkInterval*6)/unhappyBlinkInterval;
     for(int i=0;i<NUMPIXELS;i++){
        if (i%3 == t) leds.setPixelColor(i, leds.Color(0,255,0)); // red
        else leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
     }
  }
  else {
    isUnhappyLed = false;
      ledStartTime = 0;
      for(int i=0;i<NUMPIXELS;i++){
        leds.setPixelColor(i, leds.Color(0,0,0)); // off
        leds.show(); // This sends the updated pixel color to the hardware.
      }
  }
}

void checkVibration() {
  int vibrationTime = currTime - vibartionStartTime;
  if (isVibrating && vibrationTime<vibrationDuration) {
    if (vibrationTime%(vibrationInterval+vibrationRestInterval)< vibrationInterval) {
      digitalWrite(VIBRATIONPIN,HIGH);
    }
    else{
      digitalWrite(VIBRATIONPIN,LOW);
    }
  }
  else if (isVibrating){
    isVibrating = false;
    vibartionStartTime = 0;
    digitalWrite(VIBRATIONPIN,LOW);
  }
  else if (!isVibrating){
    vibartionStartTime = 0;
    digitalWrite(VIBRATIONPIN,LOW);
  }
}

void startVibration() {
  if (isVibrating) return;
  isVibrating = true;
  isUnhappyLed = false;
  isHappyLed = false; 
  vibartionStartTime = currTime;
}

void startHappyLed() {
  if (isHappyLed) return;
  isHappyLed = true;
  isUnhappyLed = false;
  isVibrating = false;
  ledStartTime = currTime;
}

void startUnhappyLed() {
  if (isUnhappyLed) return;
  isUnhappyLed = true;
  isHappyLed = false; 
  isVibrating = false;
  ledStartTime = currTime;
}


bool isButtonPressed() {
  button.update();
  Serial.println(button.pressed());
  return button.fell();
}

char getSerialData() {
  if (Serial.available()) { // if data available to read
    return Serial.read();
  }
  return '0';
}
