import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import gab.opencv.*; 
import MultiTouchKitUI.*; 
import processing.serial.*; 
import blobDetection.*; 
import processing.sound.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class sketch_16480_multiTouch extends PApplet {







int tx = 6;               //number of transmitter lines (rx)
int rx = 6;               //number of receiver lines (rx)
int serialPort = 2;       //serial port that the Arduino is connected to
int comPort = 3;          //serial port that the Arduino want to connect to
Serial comSerial;
MultiTouchKit mtk;
SoundFile whiningSound;
SoundFile barkingSound;
Puppy myPuppy;

//Tunable Parameters
int maxInputRange = 90; // if your input is too weak, choose a smaller maxInputRange
float threshold = 0.70f;  // everything bigger than this will be considered a touch
float sameBlobMargin = 0.01f;
int sameBlobTimeMargin = 50;
int tapDuration = 150 ;
int continuousTapInBetweenTime = 500;
int interactionWaitPeriod = 10000;

int[][] values;           // values used for visualization
int[][] rawvalues;        // raw values recieved from serial port
long[][] baseline;        // baseline values saved for calibartion
BlobDetection theBlobDetection;
ArrayList<ActiveBlob> currBlobs = new ArrayList<ActiveBlob>();
PImage scaledbc;
int blobNum;
boolean isTap = false;
int lastTapTime;
boolean isInteracting = false;
int continousTaps = 0;
int interactionStartTime = -1;
int interactionEndTime = -1;
int restStartTime = 0;
int restEndTime = 0;

Table tableRaw;           // table for recording raw values 
Table tableAdj;           // table for recording values used for visualization
Table tableBas;           // table for recording baseline values 

public void setup(){
  
  background(255);
  mtk = new MultiTouchKit(this,tx,rx,serialPort,500,500);      // instantiate the MultiTouchKit  
  mtk.autoDraw(true);                                          // visualize the touch points
  mtk.setMaxInputRange(maxInputRange);                         // set the brightness of touch points
  mtk.setThresh(threshold);                                    // set the threshold for blob detection
  whiningSound = new SoundFile(this, "dogWhining.mp3");
  barkingSound = new SoundFile(this, "puppyBarking.mp3");
  myPuppy = new Puppy();
  comSerial = new Serial(this, Serial.list()[comPort], 9600);
}

public void draw() {
    // values = mtk.getAdjustedValues();                            // values used for visualization
    // rawvalues = mtk.getRawValues();                              // raw values recieved from serial port
    // baseline = mtk.getBaseLine();   
    theBlobDetection = mtk.getBlobDetection();
    scaledbc = mtk.getScaledbc();  
    theBlobDetection.computeBlobs(mtk.imageForBlobDetection(scaledbc).pixels);
    blobNum = theBlobDetection.getBlobNb();
    processData();
    myPuppy.update();
    /* ========== Implementation starts here ========== */
    if (isTap()) {
      println("tap detected");
    }
    if (isPress()) {
      println("press detected");
      println(pressLasted());
    }
    if (isTouch()) {
      println("touch detected");
    }
    if (isSingleTouch()) {
      println("single touch detected");
    }
    if (isMultiTouch()) {
      println("multi touch detected");
    }
    print("# of continous taps: ");
    println(continuousTap());
    print("interaction time: ");
    println(interationLasted());
    print("rest time: ");
    println(restLasted());
}

public ActiveBlob isInActiveBlobs(Blob blob) {
  int n = currBlobs.size();
  for (int i = 0; i < n; i++) {
    ActiveBlob currBlob = currBlobs.get(i);
    if (currBlob.isSameBlob(blob)) {
      return currBlob;
    }
  }
  return null;
}

public void processData() {
  // reset isTap
  isTap = false;

  int currTime = millis();

  // clear number of countinuousTaps if no longer tapping
  if (currTime - lastTapTime > continuousTapInBetweenTime) continousTaps = 0;
  // interaction starts when blob is detected
  if (blobNum > 0 && !isInteracting){
    isInteracting = true;
    interactionStartTime = currTime;
    restStartTime = -1;
    restEndTime = -1;
  } 
  // interaction ends when blob has not been detected for a while
  if (isInteracting && blobNum==0 && interactionEndTime<(currTime-interactionWaitPeriod)){
    isInteracting = false;
    restStartTime = currTime;
    interactionStartTime = -1;
    interactionEndTime = -1;
  }
  if (blobNum>0) interactionEndTime = currTime;
  if (blobNum == 0 && !isInteracting) restEndTime = currTime;
  
  for (int i = 0; i<blobNum ; i++) {
    Blob blob = theBlobDetection.getBlob(i);
    ActiveBlob activeBlob = isInActiveBlobs(blob);
    // check if any of detected blobs are existing blobs
    if (activeBlob != null) {
      activeBlob.updateBlob(blob);
      activeBlob.updateTime(currTime);
    }
    else { 
      // instantiate & add new activeBlob
      currBlobs.add(new ActiveBlob(blob, currTime));
    }
  }
  // remove blobs that no longer appear
  for (int i = 0; i < currBlobs.size(); i++) {
    ActiveBlob currBlob = currBlobs.get(i);
    if (currBlob.getLastTime() <= currTime-sameBlobTimeMargin) { // blob disappeared
      // tap is detected when 
      // blob disaapears, nothing is being pressed
      // isTap is set to true even if other blobs are still being pressed
      if (!isPress()) {
        println(currBlob.getDuration());
        isTap = true;
        lastTapTime = currTime;
        continousTaps += 1;
      }
      currBlobs.remove(i);
      i--;
    }
  }
}

/* tap is detected when finger is up */
public boolean isTap() {
  return isTap;
}

public int continuousTap(){
  return continousTaps;
}

/* isPress() returns true when more than half of the blobs have been pressed for tapDuration */
public boolean isPress() {
  int n = currBlobs.size();
  int pn = 0;
  for (int i = 0; i < n; i++) {
    if (currBlobs.get(i).getIsPress()) pn+=1;
  }
  return pn>(n/2);
}

/* pressLasted() returns the press time of the blob that has the longest press time in milliseconds*/
public int pressLasted() {
  int n = currBlobs.size();
  int time = 0;
  for (int i = 0; i < n; i++) {
    time = max(time , currBlobs.get(i).getDuration());
  }
  return time;
}

public boolean isTouch() {
  return isTap()||isPress();
}

public boolean isSingleTouch() {
  return (currBlobs.size() == 1)&&(isTouch());
}

public boolean isMultiTouch() {
  return (currBlobs.size() > 1) && (isTouch());
}

public int interationLasted() {
  if (interactionStartTime != -1 && interactionEndTime != -1)
    return interactionEndTime-interactionStartTime;
  return 0;
}

public int restLasted() {
  if (restStartTime != -1 && restEndTime != -1)
    return restEndTime-restStartTime;
  return 0;
}

class ActiveBlob{
  Blob blob;
  int startTime;
  int duration = 0;
  int lastTime;
  boolean blobIsPress = false;

  ActiveBlob(Blob b, int t) {
    blob = b;
    startTime = t;
    lastTime = t;
  }

  public boolean isSameBlob(Blob blob2) {
    return ((abs(blob.x-blob2.x)<sameBlobMargin)
      && (abs(blob.y-blob2.y)<sameBlobMargin));
  }

  public void updateBlob(Blob b) {
    blob = b;
  }

  public void updateTime(int t) {
    lastTime = t;
    duration = t - startTime;
    if ((duration > tapDuration)) {
      blobIsPress = true;
    }
  }

  public int getLastTime() {
    return lastTime;
  }

  public int getDuration() {
    return duration;
  }

  public boolean getIsPress() {
    return blobIsPress;
  }
}

class Puppy{
  int leaveTooLongTime = 15000; //15s for demo purpose
  int playTooLongTime = 15000; //15s for demo purpose
  int rubTapCounts = 20;

  Puppy() {

  }

  public void update() {
    if (restLasted()>leaveTooLongTime) { // leave alone for too long
      if (!whiningSound.isPlaying()) whiningSound.loop();
      //pushed lightly for attention -> vibration
      comSerial.write('v');    
    }
    if (interationLasted() > 0) {
      if (isMultiTouch()) { // gently pad
        // calms by turning off sounds
        if (whiningSound.isPlaying()) whiningSound.pause();
        if (barkingSound.isPlaying()) barkingSound.pause();
      }
      if (continuousTap() > rubTapCounts || isPress()){ // rub ears
        if (!barkingSound.isPlaying()) barkingSound.play();
      }
      if (isSingleTouch()) { // play with her
        // calms by turning off sounds
        if (whiningSound.isPlaying()) whiningSound.pause();
        if (barkingSound.isPlaying()) barkingSound.pause();
        // enjoys it and is temporarily distracted  -> led
        comSerial.write('h');
      }
      // this part of code should be in arduino
      // if () { // rub her belly
      //   // enjoys it and is temporarily distracted -> led
      // }
      if (interationLasted() > playTooLongTime) { // play for too long
        //  impatient and avoid us again -> led
        comSerial.write('i');
      }
    }
  }
}
  public void settings() {  size(500,500); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "--present", "--window-color=#666666", "--stop-color=#cccccc", "sketch_16480_multiTouch" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
